﻿Option Explicit On
Option Strict On
Imports System.Data
Imports System.Data.Sql
Imports System.Data.SqlClient
Imports System.Data.SqlTypes
Imports System.Xml
Imports System.Text
Imports System.IO

Public Class XQuery
    'Procedures executed during the Form_Load event.(and a couple of others)
    Private cnNwind As SqlConnection
    Private cmNwind As New SqlCommand
    Private strIEFile As String = "\Program Files\Internet Explorer\Iexplore.exe"
    Private blnEnableIE As Boolean
    Private strNS As String
    Private blnTyped As Boolean
    Private strXmlColName As String
    Private intXmlCol As Integer
    Private alCustomerIDs As New ArrayList
    Private alOrderIDs As New ArrayList
    Private blnXML1Exists As Boolean
    Private blnXML2Exists As Boolean
    Private lngBytes As Long
    Private intRows As Integer
    Private dblTime As Double
    Private blnBypassHandler As Boolean
    Private blnHasLoaded As Boolean
    Private objTimer As Stopwatch
    Private sbXML As StringBuilder

    Private Sub XQuery_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim strConn As String = Nothing
        Try
            If File.Exists(strIEFile) Then
                blnEnableIE = True
            End If
            'Set the connection
            strConn = My.Settings.NorthwindConnection
            cnNwind = New SqlConnection(strConn)
            cmNwind.Connection = cnNwind
            cmNwind.CommandType = CommandType.Text
            cnNwind.Open()
            Dim blnDropTables As Boolean = My.Settings.DropOrdersXMLOnStart

            Dim strSQL As String
            If My.Settings.CleanTablesOnStart Then
                'Following is required to prevent overlong row error messages
                strSQL = "DBCC CLEANTABLE ('Northwind', 'Orders'); "
                cmNwind.CommandText = strSQL
                cmNwind.ExecuteNonQuery()
            End If

            If blnDropTables Then
                'Remove Orders table columns and SchemaCollections, if present (clean slate)
                Dim intCtr As Integer
                Dim strMsg As String = Nothing
                Dim strColName As String = Nothing
                Dim strTableName As String = Nothing
                cmNwind.CommandTimeout = 30
                For intCtr = 0 To 1
                    Select Case intCtr
                        Case 0
                            strTableName = "Orders"
                            strColName = "OrderXML1"
                        Case 1
                            strTableName = "Orders"
                            strColName = "OrderXML2"
                    End Select

                    'Drop existing indexes to allow dropping columns
                    DropXMLIndexes(strTableName, strColName, True)

                    strSQL = "ALTER TABLE " + strTableName + " DROP COLUMN " + strColName + "; "
                    cmNwind.CommandText = strSQL
                    strMsg += strSQL + vbCrLf
                    Try
                        cmNwind.ExecuteNonQuery()
                    Catch exc As Exception
                        strMsg += exc.Message + vbCrLf
                    End Try
                    strSQL = "DROP XML SCHEMA COLLECTION " + strColName + "SchemaColl; "
                    cmNwind.CommandText = strSQL
                    strMsg += strSQL + vbCrLf
                    Try
                        cmNwind.ExecuteNonQuery()
                    Catch exc As Exception
                        strMsg += exc.Message + vbCrLf
                    End Try
                Next
            Else
                If cnNwind.State <> ConnectionState.Open Then
                    cnNwind.Open()
                End If
                TestSchemaCollections("Order")
                chkOrderXML1Index.Enabled = True
                chkOrderXML2Index.Enabled = True
                'Set Orders table index check boxes
                GetTableXmlIndexes("Orders")
            End If

            'Populate OrderID array list
            strSQL = "SELECT OrderID FROM Orders ORDER BY OrderID"
            cmNwind.CommandText = strSQL
            Dim sdrOrders As SqlDataReader = cmNwind.ExecuteReader
            With sdrOrders
                If .HasRows Then
                    While .Read
                        alOrderIDs.Add(.GetValue(0))
                    End While
                End If
                .Close()
            End With
            cnNwind.Close()
            'Create the xml columns
            If My.Settings.DropOrdersXMLOnStart Then
                AddOrderXmlColumns()
            End If
            txtOrderSQL.Text = "-- Click Fill OrderXML1 and Fill OrderXML2 to populate the Orders table's xml columns." + vbCrLf + vbCrLf + _
             "-- Mark a Schema check box to add an XmlSchemaCollection for XML document validation." + vbCrLf + vbCrLf + _
             "-- Mark an Index check box to add a primary XML index and, optionally, one or more secondary XML indexes to the xml column." + vbCrLf + vbCrLf + _
             "-- Select a sample query or Clear to enable the Execute XQuery Expression button."
        Catch exc As SqlException
            Application.UseWaitCursor = False
            If exc.Message.Contains("Cannot open database") Then
                Dim strMsg As String = "The '" + strConn + "' connection string can't " + _
                 "open the 'Northwind' database. Open app.config and change the " + _
                 "'connectionString' attribute value to point to your SQL Server 2005 " + _
                 "or SQL Express instance with the required database."
                MsgBox(strMsg, MsgBoxStyle.Exclamation, "Failed to Open the Northwind Database")
            Else
                MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Exception During Load Event")
            End If
        Catch exc As Exception
            Application.UseWaitCursor = False
            MsgBox(exc.Message)
        Finally
            If cnNwind IsNot Nothing Then
                cnNwind.Close()
            End If
            blnHasLoaded = True
            Application.UseWaitCursor = False
        End Try
    End Sub

    Private Function DropXMLIndexes(ByVal strTableName As String, ByVal strColName As String, ByVal blnIsOpen As Boolean) As String
        'Drop all four XML indexes
        Dim intIndex As Integer
        Dim strSQL As String = Nothing
        Dim strDrop As String = Nothing
        If cnNwind.State <> ConnectionState.Open Then
            cnNwind.Open()
        End If
        If strTableName = "SalesOrders" Then
            strColName = Replace(strColName, "Order", "SalesOrder")
        End If
        For intIndex = 0 To 3
            Select Case intIndex
                Case 0
                    strDrop = "DROP INDEX sidx_prop_" + strColName + " ON " + strTableName + "; "
                Case 1
                    strDrop = "DROP INDEX sidx_value_" + strColName + " ON " + strTableName + "; "
                Case 2
                    strDrop = "DROP INDEX sidx_path_" + strColName + " ON " + strTableName + "; "
                Case 3
                    'Primary key index must be dropped last
                    strDrop = "DROP INDEX pidx_" + strColName + " ON " + strTableName + "; "
                Case 4
                    'Temporary for wrong index created
                    strColName = Replace(strColName, "Order", "SalesOrder")
                    strDrop = "DROP INDEX pidx_" + strColName + " ON " + strTableName + "; "
            End Select
            cmNwind.CommandText = strDrop
            Try
                cmNwind.ExecuteNonQuery()
                strDrop += " (Dropped)"
            Catch exc As Exception
                'Missing indexes
                strDrop += " (Missing)"
            End Try
            strSQL += strDrop + vbCrLf
            If strTableName.Contains("Orders") Then
                If strColName.Contains("XML1") Then
                    chkOrderXML1IndexPath.Checked = False
                    chkOrderXML1IndexValue.Checked = False
                    chkOrderXML1IndexProp.Checked = False
                Else
                    chkOrderXML2IndexPath.Checked = False
                    chkOrderXML2IndexValue.Checked = False
                    chkOrderXML2IndexProp.Checked = False
                End If
            End If
        Next
        If Not blnIsOpen Then
            cnNwind.Close()
        End If
        Return "-- Click Fill " + strColName + " to repopulate the column" + vbCrLf + strSQL
    End Function

    Private Sub TestSchemaCollections(ByVal strColName As String)
        'Test for existence of XML1 and XML1 SchemaCollections
        'Expects an open connection
        Dim intCtr As Integer
        Dim strSQL As String
        Try
            For intCtr = 1 To 2
                strSQL = "SELECT xml_schema_namespace(N'dbo',N'" + strColName + "XML1SchemaColl')"
                strSQL += ".query('/xs:schema[@targetNamespace=""http://www.northwind.com/schemas/" + strColName + """]')"
                If intCtr = 2 Then
                    strSQL = Replace(strSQL, "XML1", "XML2")
                End If
                cmNwind.CommandText = strSQL
                blnBypassHandler = True
                Dim xrSchema As XmlReader = Nothing
                Try
                    'Read the schema, if present
                    xrSchema = cmNwind.ExecuteXmlReader
                    'cnNwind.Close()
                    If xrSchema Is Nothing Then
                        If intCtr = 2 Then
                            chkOrderXML2Schema.Checked = False
                        Else
                            chkOrderXML1Schema.Checked = False
                        End If
                    Else
                        If intCtr = 2 Then
                            chkOrderXML2Schema.Checked = True
                        Else
                            chkOrderXML1Schema.Checked = True
                        End If
                    End If
                    xrSchema.Close()
                Catch exc As Exception
                    chkOrderXML1Schema.Checked = False
                Finally
                    blnBypassHandler = False
                    If xrSchema IsNot Nothing Then
                        xrSchema.Close()
                    End If
                End Try
            Next intCtr
        Catch exc As Exception
            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "TestSchemaCollections Exception")
        End Try
    End Sub

    Private Sub GetTableXmlIndexes(ByVal strTableName As String)
        'Use lighter-weight sys.indexes for changes between Orders and SalesOrders
        'Expects an open connection
        Dim strSQL As String = "SELECT name FROM sys.indexes WHERE type_desc = 'XML'; "
        Dim rdrData As SqlDataReader = Nothing
        Dim strIndexes As String = Nothing
        cmNwind.CommandText = strSQL
        Try
            rdrData = cmNwind.ExecuteReader()
            With rdrData
                If .HasRows Then
                    While .Read
                        strIndexes += .GetString(0) + ", "
                    End While
                    .Close()
                    SetIndexCheckBoxes(strTableName, strIndexes)
                Else
                    'Error
                End If
            End With
        Catch exc As Exception
            'Error
        Finally
            If Not rdrData.IsClosed Then
                rdrData.Close()
            End If
        End Try
    End Sub

    Private Sub SetIndexCheckBoxes(ByVal strTableName As String, ByVal strIndexes As String)
        Dim strName As String = Nothing
        If strTableName = "SalesOrders" Then
            strName = "SalesOrder"
        ElseIf strTableName = "Orders" Then
            strName = "Order"
        ElseIf strTableName = "Customers" Then
            strName = "Customer"
        End If
        blnBypassHandler = True
        'Set XML1 index check boxes
        If strIndexes.Contains("pidx_" + strName + "XML1") Then
            chkOrderXML1Index.Checked = True
        Else
            chkOrderXML1Index.Checked = False
        End If
        If strIndexes.Contains("sidx_path_" + strName + "XML1") Then
            chkOrderXML1IndexPath.Checked = True
        Else
            chkOrderXML1IndexPath.Checked = False
        End If
        If strIndexes.Contains("sidx_value_" + strName + "XML1") Then
            chkOrderXML1IndexValue.Checked = True
        Else
            chkOrderXML1IndexValue.Checked = False
        End If
        If strIndexes.Contains("sidx_prop_" + strName + "XML1") Then
            chkOrderXML1IndexProp.Checked = True
        Else
            chkOrderXML1IndexProp.Checked = False
        End If
        'Set XML2 index check boxes
        If strIndexes.Contains("pidx_" + strName + "XML2") Then
            chkOrderXML2Index.Checked = True
        Else
            chkOrderXML2Index.Checked = False
        End If
        If strIndexes.Contains("sidx_path_" + strName + "XML2") Then
            chkOrderXML2IndexPath.Checked = True
        Else
            chkOrderXML2IndexPath.Checked = False
        End If
        If strIndexes.Contains("sidx_value_" + strName + "XML2") Then
            chkOrderXML2IndexValue.Checked = True
        Else
            chkOrderXML2IndexValue.Checked = False
        End If
        If strIndexes.Contains("sidx_prop_" + strName + "XML2") Then
            chkOrderXML2IndexProp.Checked = True
        Else
            chkOrderXML2IndexProp.Checked = False
        End If
        blnBypassHandler = False
    End Sub

    Private Sub AddOrderXmlColumns()
        Try
            txtOrderSQL.Text = ""
            blnBypassHandler = True
            chkOrderXML1Schema.Checked = False
            chkOrderXML1Index.Checked = False
            chkOrderXML2Schema.Checked = False
            chkOrderXML2Index.Checked = False
            Dim strSQL As String = "ALTER TABLE Orders ADD OrderXML1 xml NULL, OrderXML2 xml NULL; "
            cmNwind.CommandText = strSQL
            cnNwind.Open()
            Dim intRetVal As Integer = cmNwind.ExecuteNonQuery
            If intRetVal = -1 Then
                txtOrdersXQResult.Text = ""
                OrdersControlState(True)
            Else
                'Error
            End If
            cnNwind.Close()
            blnBypassHandler = False
        Catch exc As Exception
            If exc.Message.Contains("failed") Then
                txtOrdersXQResult.Text = "Click Add xml Columns to correct the problem"
            Else
                txtOrdersXQResult.Text = exc.Message
            End If
        Finally
            cnNwind.Close()
        End Try
    End Sub

    Private Function DropXmlSchemaCollection(ByVal strTableName As String, ByVal strColName As String) As String
        Try
            txtOrdersXQResult.Text = ""
            DropXMLIndexes(strTableName, strColName, False)
            Dim strMsg As String = Nothing
            'Invoke commands separately to avoid problems if column is missing
            Dim strSQL1 As String = "ALTER TABLE " + strTableName + " DROP COLUMN " + strColName + "; "
            Dim strSQL2 As String = "DROP XML SCHEMA COLLECTION " + strColName + "SchemaColl; "
            Dim strSQL3 As String = "ALTER TABLE " + strTableName + " ADD " + strColName + " xml NULL;"
            cnNwind.Open()
            Dim intCtr As Integer
            For intCtr = 1 To 3
                If intCtr = 1 Then
                    cmNwind.CommandText = strSQL1
                    strMsg += strSQL1
                ElseIf intCtr = 2 Then
                    cmNwind.CommandText = strSQL2
                    strMsg += strSQL2
                ElseIf intCtr = 3 Then
                    cmNwind.CommandText = strSQL3
                    strMsg += strSQL3
                End If
                strMsg += vbCrLf
                Try
                    cmNwind.ExecuteNonQuery()
                Catch ex As Exception
                    strMsg += ex.Message + vbCrLf
                End Try
            Next intCtr
            cnNwind.Close()
            Return "-- Click Fill " + strColName + " to repopulate the column" + vbCrLf + strMsg
        Catch exc As Exception
            Return exc.Message
        Finally
            cnNwind.Close()
        End Try
    End Function

    Private Sub OrdersControlState(ByVal blnEnabled As Boolean)
        'Disable controls curing fill
        btnFillOrderXML1.Enabled = blnEnabled
        btnFillOrderXML2.Enabled = blnEnabled
        chkOrderXML1Schema.Enabled = blnEnabled
        chkOrderXML2Schema.Enabled = blnEnabled
        chkOrderXML1Index.Enabled = blnEnabled
        chkOrderXML1IndexPath.Enabled = blnEnabled
        chkOrderXML1IndexValue.Enabled = blnEnabled
        chkOrderXML1IndexProp.Enabled = blnEnabled
        chkOrderXML2Index.Enabled = blnEnabled
        chkOrderXML2IndexPath.Enabled = blnEnabled
        chkOrderXML2IndexValue.Enabled = blnEnabled
        chkOrderXML2IndexProp.Enabled = blnEnabled
    End Sub

    Private Sub chkBytes_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBytes.CheckedChanged
        If chkBytes.Checked Then
            lblRows.Text = "KChars:"
            lblRowsPerSec.Text = "KC/Sec:"
            If txtTime.Text.Length > 0 Then
                Dim dblKB As Double
                dblKB = lngBytes / 1000
                If dblKB > 1.0 Then
                    txtRows.Text = dblKB.ToString("#0.0")
                Else
                    txtRows.Text = dblKB.ToString("0.000")
                End If
                txtRowsPerSec.Text = (dblKB / dblTime).ToString("#0.0")
            End If
        Else
            lblRows.Text = "Rows:"
            lblRowsPerSec.Text = "Rows/Sec:"
            If txtTime.Text.Length > 0 Then
                txtRows.Text = intRows.ToString
                txtRowsPerSec.Text = (intRows / dblTime).ToString("#0.0")
            End If
        End If
    End Sub
End Class
